;
; IDL Event Callback Procedures
; MSP_BDP_GUI_eventcb
; 
; v3.4.6 DPC  20220622 - fixed bad triage of LLQ A/D problems
; v3.4.5 DPC  20200118 - worked on ski jump detection to reduce number of deleted images
; v3.4.4 DPC  20170609 - fixed FF zero hop testing logic for multiple FF sets in a run
; v3.4.3 DPC  20161206 - fixed minor bug in linearity code (S6) and turned off print statements
; v3.4.2 DPC  20140414 - added IZYJ-Wide filter type
; v3.4.1 DPC  20131220 - rewrote bad pixel mask creation (deprecated histogram analysis)
; v3.4.0 DPC  20130920 - added NCOADDS support
; v3.3.1 DPC  20130125 - minor bug fixes in coadd-with-median-sub
; v3.3.0 DPC  20121031 - added LL Quad bad test (no repair, just flag bad)
; v3.2.3 DPC  20111218 - added zero hop fix by dither to P12
; v3.2.2 DPC  20111214 - flat field support for IZYJ-Spec
; v3.2.1 DPC  20111103 - bug fix on S12 K-pol ZHT
; v3.2.0 DPC  20111031 - added K-Pol flat support (S10, S12) with ZHT testing
; v3.1.3 DPC  20110728 - just cuz
; v3.1.2 DPC  20110424 - fixed bug in S5 BPM calc
; v3.1.1 DPC  20101224 - bug fixes for H17, IXYJ-X spec; others
; v3.1.0 DPC  20101124 - updated for new filter wheel population plan and new modes
; v3.0.4 DPC  20100824 - minor bug fix in directory processor
; v3.0.3 DPC  20100728 - fixed bugs in dir change option
; v3.0.2 DPC  20100708 - added PCal capability, fixed minor RAW1 jump bugs
; v3.0.1 DPC  20100619 - many minor bug fixes
; v3.0.0 DPC  20100608 - new FITS header variables; added directory change option through new roots
; v2.1.5 DPC  20100209 - added ignore ZHT button to S12
; v2.1.4 DPC  20100204 - changed zero hop probability determinations
; v2.1.3 DPC  20100128 - minor bug fix
; v2.1.2 DPC  20100125 - all outputs single precision, tables all support "skip" keyword
; v2.1.1 DPC  20100114 - added rollback to original images in P12
; v2.1.0 DPC  20100111 - save mean_PA for polar flats, read same for testing pol groups at S12
; v2.0.1 DPC  20091228 - added zero hop group count and list textboxes to S12/pol
; v2.0.0 DPC  20091223 - added new tests, fixes in pol flats and pol groups for zero hopping problem
; v1.5.3 DPC  20091217 - turned off flat flattenting, tightened central normalization zone
; v1.5.2 DPC  20091202 - now works for K17-Spec mode
; v1.5.1 DPC  20091201 - fixed ski-jump problems with column clocking correction
; v1.5  DPC  20091130 - flattened phot, pol flats before application using model_background
; v1.3  DPC  20080930 - added HK-Spec recognition
;
;
; Empty stub procedure used for autoloading.
;
pro MSP_BDP_GUI_eventcb
end
;
;-------------------------------------------------------------------------
pro MSP_BDP_PostCreation, wWidget, _EXTRA=_VWBExtra_
    ;
    ; if windows and idl8 select font
    ; 
    if(STRTRIM(STRUPCASE(!VERSION.OS_FAMILY),2) eq 'WINDOWS') then begin
      WIDGET_CONTROL, DEFAULT_FONT='Tahoma*13' ; 'Veranda*15'
    endif
    ;
    ;   set up and clear summary structure
    ;
    host = GETENV('computername')
    if(STRLEN(host) eq 0) then host = GETENV('HOSTNAME')
    ;
    version_short_string = 'v3.4.6 20220622 on '+host
    version_string = 'Mimir - Basic Data Processing (MSP-BDP) '+version_short_string
    ;
    summary_struct={run_name:       'default',      $         ; name of the run in YYYYMM form
                    last_date:       '',     		$         : date of last processing in systime() form
                    who:            'Dan Clemens',  $         ; person who performed last processing
                    root_raw:        '.',           $         ; root directory for raw data
                    root_reduced:    '.',           $         ; root directory for reduced data
                    max_step:        13,			$		  ; final processing step number (TBD)
                    num_nights:      0L,            $         ; number of nights in this run
                    night_date:      lonarr(100),   $         ; night designation of each night
                    night_images:    lonarr(100),   $         ; number of images in each night
                    night_step:      intarr(100),   $         ; processing step number for each night
                    night_comment:   strarr(100),   $         ; comment to add to each night's processing summary
					night_who:		 strarr(100),	$		  ; reducer for the most recent level for this night
					version_short_string: version_short_string, $
					version_string:	 version_string,$		  ; as above
                    ;
                    echo_window_wid: -1L,			$		  ; wid of the bottom text window for messages
                    backup_file:	 '',            $		  ; pathname of the backup summary file (copy of previous open)
                    log_file:        '',            $         ; pathname of the logging file
                    lu_log:			 0,				$		  ; logical unit of log file
                    default_font:	'Tahoma*14', $		  ; the default font
                    ;
                    row513_date:	20060606L,		$			; date after which row513 works OK - is date right?
                    ;
                    ;	P1 specific
                    ;
                    P1_button_wid:	lonarr(8),		$		  ; wids of buttons on P1 tab
                    ;
                    ;	P2 specific
                    ;
                    P2_button_wid:	lonarr(10),		$		  ; wids of buttons on P2 tab
                    P2_LED_wid:		lonarr(4),		$		  ; wids of LEDs on P2 tab
                    P2_droplist_night_wid:	-1L,	$	  	  ; wid of the droplist of night designations
					;
                    P3_button_wid:  lonarr(6),		$		  ; wids of buttons/droplist on P3 tab
                    	P3_column_flag:		0L,		$		  ; yes/no flag of whether to perform col check
                    P3_window_wid:  lonarr(4),      $         ; wids of text return windows
                    P3_LED_wid:     lonarr(2),      $         ; wids of LEDs on P3 tab
                    P3_LED_colors:	strarr(25),		$
                    P3_droplist_night_wid:  -1L,    $		  ; wid of droplist
                    num_kokopeli:			0L,		$		  ; number of images listed in the kokopeli structure
					fix_kokopeli:			0L,		$		  ; number of images with FIX recommendations
					num_column:				0L,     $		  ; number of images with column clock errors
					;
					P4_button_wid:  lonarr(13),     $
					P4_draw_window_id:      0,      $
					P4_LED_wid:     lonarr(2),      $
					P4_LED_colors:	strarr(25),		$
					P4_droplist_night_wid:  -1L,    $
					;
					P5_button_wid:	lonarr(12),		$
					P5_draw_window_id:		0,		$
					P5_LED_wid:		lonarr(1),		$
					P5_droplist_night_wid:	lonarr(2),	$
					P5_base_wid:	lonarr(4),		$
					P5_text_wid:	lonarr(4),		$
					P5_LED_colors:	strarr(10),		$
					;
					P6_button_wid:	lonarr(14),		$
					P6_draw_window_id:		0,		$
					P6_LED_wid:		lonarr(4),		$
					P6_droplist_wid:lonarr(7),		$
					P6_text_wid:	lonarr(2),		$
					P6_LED_colors:	strarr(25),		$
					P6_tab_wid:				0L,		$
					P6_current_tab:			0L,		$
					;
					P7_button_wid:	lonarr(2),		$
					P7_LED_wid:		lonarr(1),		$
					P7_droplist_night_wid:lonarr(1),$
					P7_text_wid:	lonarr(1),		$
					P7_LED_colors:	strarr(15),		$
					;
					P8_button_wid:	lonarr(3),		$
					P8_LED_wid:		lonarr(1),		$
					P8_droplist_night_wid:lonarr(1),$
					P8_text_wid:	lonarr(1),		$
					P8_LED_colors:	strarr(105),	$
					;
					P9_button_wid:	lonarr(3),		$
					P9_LED_wid:		lonarr(1),		$
					P9_LED_colors:	strarr(25),		$
					;
					P10_button_wid:	lonarr(20),		$
					P10_draw_window_id:lonarr(2),	$
					P10_text_wid:			0L,		$
					P10_LED_wid:	lonarr(2),		$
					P10_LED_colors:	strarr(25),		$
					P10_tab_wid:			0L,		$
					P10_current_tab:		0L,		$
					;
					P11_button_wid:	lonarr(6),		$
					P11_LED_wid:	lonarr(3),		$
					P11_text_wid:	lonarr(1),		$
					P11_LED_colors:	strarr(15),		$
					;
					P12_button_wid:	lonarr(8),		$
					P12_base_wid:			0L, 	$	rollback base
					P12_text_wid:	lonarr(2),		$
					;P12_draw_window_id:		0,		$
					P12_droplist_night_wid:lonarr(1),$
					P12_LED_wid:	lonarr(2),		$
					P12_LED_colors:	strarr(105),		$
					P12_tab_wid:			0L,		$
					P12_current_tab:		0L,		$
					P12_checkbox_wid:		0L,		$
						P12_Ignore_ZHT_flag:	1L,		$         ; set to 1 to override ZHT testing - DPC  20151208
					P12_radio_dither_wid:	0L,		$
						P12_zht_dither_flag:0L,		$
                    ;
                    ;	buffer for current FITS header
                    ;
                    raw_header: strarr(300) }				  ; the raw FITS header
    ;
    ;	structure for capturing parsed FITS header variables
    ;
    FITS_struct =  {    image_id:	0L,		$			; image number from observations
						naxis1:		0,		$			; number of columns in image
						naxis2:		0,		$			; number of rows in image
						observer:	'',		$			; name(s) of observers
						CDS:		1,		$			; 1 for CDS, 0 for non-CDS
						LST_obs:	0.0d0,	$			; LST, in decimal hours
						ra_tele:	0.0d0,	$			; RA of the telescope, in radians
						dec_tele:	0.0d0,	$			; Dec of telescope, in radians
						airmass:	0.0,	$			; Airmass
						instrument:	'',		$			; Name of instrument
						filter:		strarr(5),$			; Filter names for up to 5 filter wheels
						raw_filename:	'',	$			; Image Filename as originally obtained
						objname:	'',		$			; Object Name
						object:		'',		$			; Object
						date_obs:	'',		$			; Date, time of observation (string)
						exptime:	0.0,	$			; Exposure time, in seconds
						ut:			0.0d0,	$			; UT at start of exposure, in decimal hours
						numcoadds:	1,		$			; Number of Coadds making up this exposure
						;
						;	new 20100608
						;
						header_version:	0L,	$			; FITS header version: 0 = pre 2010; 1 = 2010 +
						camera_pos:	0L,		$			; Camera position, in steps from home
						slit_pos:	0L,		$			; Slit position, in detents from home
						decker_pos:	0L,		$			; Decker position, in detents from home
						hwp_step:	0L,		$			; HPW position, in steps
						hwp_ang:   0.0,		$			; HWP position angle, in degrees
						hwp_home:	0L,		$			; HWP home flag
						;
						comment:	strarr(25) }		; Comment blockimage_number:		   '',  $		  ; full image number identifier -- TBD
    ;
    max_image = 9000L
    header_list = REPLICATE(FITS_struct,max_image)
    ;
    group_struct = {   group_id:			    0,  $		  ; consective number of this group (0 start)
                       group_images:            0,  $         ; number of images in this group
                       group_start:             0,  $         ; first image number of images in this group
                       group_end:               0,  $         ; last image number of images in this group
                       group_list:   intarr(1000),  $         ; list of all image numbers in this group
                       group_object:           '',  $		  ; object name for images in this group
                       group_type:             '',  $         ; image type for this group
                       group_filter:           '',  $         ; filter designation for this group
                       group_exptime:         0.0,  $         ; exposure time (or -1.0 if many, ie linearity run)
                       group_frame:            '',  $         ; full frame or NCxNR subframe format
                       group_readout_mode:      0,  $         ; group readout mode
                       group_camera:           '',  $         ; camera mode for group
                       group_coadds:			1   $		  ; number of coadds in each image in the group
                     }
    ;
    noel = 0
    if(noel) then mgr = 500 else mgr = 250
    ;
    groups = REPLICATE(group_struct,mgr)					  ; up to 100 groups for each night
    ;
    night_struct = {   night_desig:			   0L,	$		  ; designation of the current night with data in this structure
					   max_image: 		max_image, 	$		  ; memory sized for this maximum number of images
    			       cds_ok:				   	0,  $		  ; flag to indicate that the CDS_RAW directory was found and was not empty
    			       raw1_ok:				    0,  $		  ; same for RAW1_RAW directory
    			       ;cds_pathname:		   '',  $		  ; pathname to CDS_RAW directory
    			       ;raw1_pathname:		   '',	$		  ; pathname to RAW1_RAW directory
                       num_images:	           0L,	$         ; number of images in the current structure
                       ;
                       paths_cds_root:  strarr(10), $		  ; root CDS directories for different processing levels
                       paths_raw1_root: strarr(10), $		  ; same, for RAW1 directories
                       ;									  ; N.B> both are *relative* to (*summary).root_raw, if
                       ;									  ;   the index = 0, and (*summary).root_reduced, if > 0
                       ;									  ;
                       image_root_index: lonarr(max_image), $ ; indices for each image for root CDS, RAW1 selection
                       ;
                       image_list:  intarr(max_image),	$	  ; vector of image numbers
                       ;
                       num_groups:			   0L,  $		  ; number of identified groupings of images
                       max_groups:          mgr-1,  $         ; maximum space for groups
                       groups:             groups,	$		  ; group structure for this night
                       ;
                       ;	keywords for image table
                       ;
                       image_name:		strarr(max_image),	$		; image names
                       image_object:	strarr(max_image),  $		; object name
                       image_type:		strarr(max_image),	$		; image type - "Image", "Bias", "Dark", "Flat-On", "Flat-Off", "Pol", "Spec"
                       image_filter:	strarr(max_image),	$		; simple filter summary (e.g., "H", "H-Pol", "JHK", "LM" etc.)
                       image_exptime:	fltarr(max_image),	$		; exposure time, in seconds
                       image_HWP_angle:	fltarr(max_image),	$		; HWP angle, in degrees
                       image_ski_jump:	intarr(max_image),	$		; two decimal integers: 00 = TB, so 11 = dual ski jump, 01=bottom ski jump only
					   image_frame:		strarr(max_image),  $		; "Full" or "NC x NR" frame format
					   image_camera:	strarr(max_image),	$		; camera type - F5, F17, PV
					   image_readout_mode:	intarr(max_image),$		; 1 = "sweet spot"; 0 = "deep wells"
                       image_comment:	strarr(max_image),	$		; one line of comment, not included above
                       ;
                       header_list: header_list			 ,  $		; vector of structures, each holding one image header
					   ;
					   image_coadds:    intarr(max_image)   $		; number of coadds in image	 (added in v3.4 20130920)
				   }
	;
	;	kokopeli analysis - save only information on images for one night needed kokopeli shifting
	;
	kokopeli_struct = {	night:					0L,	$			; night designator
						group:					0L, $			; group designator
						image:					0L, $			; image designator
						image_pathname:			'', $			; full pathname to raw CDS image
						recommend:				'', $			; shift recommendation by view/edit stage
						LL:						0,  $			; indicated shift for LL quad
						LR:						0,  $
						UL:						0,  $
						UR: 					0,  $
						sLL:					'', $			; "grade" representing shift detection knowledge
						sLR:					'', $
						sUL:					'', $
						sUR:					''  $
						}
	k_struct = REPLICATE(kokopeli_struct, 5000)
	;
	;	column repeat analysis test
	;
	column_struct = {night:					0L,	$			; night designator
					 group:					0L, $			; group designator
					 image:					0L, $			; image designator, or -1 if already fixed
					 image_pathname:		'', $			; full pathname to raw CDS image
					 col_code:		intarr(12)  $			; quad, col0, col1 ids ; all zeros for fixed
				 	 }
	c_struct = REPLICATE(column_struct, 5000)
	;
	;	RAW1 jump analysis structures
	;
	raw1_detect = {image_no: 0L, mean_even: fltarr(4), s_mean_even: fltarr(4),$
					mean_odd: fltarr(4), s_mean_odd: fltarr(4)}
	;
	RAW1_struct = {     detect: REPLICATE(raw1_detect, max_image),  $
					    position:    [0,0,0,0], $       ; (x0,y0),(x1,y1) device corners of plot-data region
				   		max_image: 			7000L, 	$		  ; me
				   		n_plot: 			0L, $
				   		x_plot:  lonarr(max_image), $
				   		y_plot:  fltarr(max_image),	$
				   		xx:			     	0L, $		; cursor x position in device coords
				   		yy:				 	0L, $
				   		xmax0:			 	0L, $
				   		xmin0:			 	0L, $
				   		xmax:			 	0L, $
				   		xmin:			 	0L, $
				   		ymax:            0.0d0, $
				   		ymin:            0.0d0, $
				   		end_frame:		    0L, $
				   		start_frame:        0L, $
				   		group_flag:			 0, $		; if set, paints group colors first as plot background
				   	 	color_names: ["black", "white", "red", "orange", "yellow", "green", $
										"blue", "violet", "gray", $
										"skyblue", "grey68", "lawngreen", "pink", "plum", $
										"slateblue", "khaki light", "khaki dark", "goldenrod light",$
										"goldenrod dark", "rosybrown light", "rosybrown dark", $
										"palegreen light", "palegreen dark", "steelblue light", "steelblue dark"], $
						color_rgb:  [ 	[0, 	0, 	0], $ 	; black
				   						[255,255,255], $ 	; white
				   						[255,0,	0], $ 		; red
				   						[255,165,0], $ 		; orange
									   	[255,255,0], $ 		; yellow
					              	   	[0,	255,0], $ 		; green
					              	   	[0,	0,	255], $ 	; blue
					              	   	[148,0,  211], $ 	; violet
					              	   	[220,220,220], $ 	; gray
					              	   	[135,206,235], $ 	; skyblue
					              	   	[173,173,173], $ 	; grey68
					              	   	[124,252,0], $ 		; lawngreen
					              	   	[255,20, 147], $ 	; pink
					              	   	[221,160,221], $ 	; plum
					              	   	[106,90, 205], $ 	; slateblue
					              	   	[255,246,143], $ 	; khaki light
					              	   	[139,134, 78], $ 	; khaki dark
					              	   	[255,193, 37], $ 	; goldenrod light
					              	   	[139,105, 20], $ 	; goldenrod dark
					              	   	[255,193,193], $ 	; rosybrown light
					              	   	[139,105,105], $ 	; rosybrown dark
					              	   	[154,255,154], $ 	; palegreen light
					              	   	[84,	139, 84	],  $ ; palegreen dark
					              	   	[99	,184,255], $	; steelblue light
					              	   	[54,100,139]],$		; steelblue dark
					              	   	;
					     jump_flags:   lonarr(2,max_image),  $		; first element is image number, second is jump flag
					     											;   flag values: 0=no jump found; 1=jump found, not yet fixed;
					     											;                2=jump fixed
					     deviation_RAW1: dblarr(2, 4, max_image) $	; the deviations, organized by odd/even
					     											; and quadrant of array
					    }
	;
	bpm_max = 5000L
	this_mask  = 	{   serial_no:					0L,     $	; serial number of this BPM
					    image_name:					'',     $	; image name of the BPM fits file
					    ut:						 0.0d0,		$	; decimal hours of UT for first image
					    ;
					    histo_x:			fltarr(256),	$	; histo x vector for this BPM
					    histo_y:			fltarr(256),	$
					    low_cut:					0.0,	$	; low cut used on this BPM
					    high_cut:					0.0,	$
					    n_bad_pixels:				 0L,	$	; the number of pixels set to 0
					    ;
					    night_desig:				 0L,	$	; the night designation of the group used to build the BPM
					    group_id:				     0L,	$	; the group number of the group used to build the BPM
					    n_images:					 0L,	$	; the number of images used to build the BPM
					    images_included:	strarr(128)		$	; the pathnames of the images used to build the BPM
					 }

	BPM_struct = 	{	bpm_max:				bpm_max,	$	; maximum number of BPM entries allowed
						;
						bpm_n_BPM:				      0, 	$	; the number of BPM entries filled
						bpm_directory:		 		 '', 	$	; relative to the reduced_root directory
						bpm_image_name:	strarr(bpm_max),	$	; vector of image names of the BPMs in this directory
						bpm_UT:			dblarr(bpm_max),	$	; UTs of each BPM file in this directory
						bpm_JD:			dblarr(bpm_max),	$	; full Julian date (YYMMDDHHMMSS) of this obs
						bpm_night:		lonarr(bpm_max),	$
						;
						bpm_mode:					  0,	$	; 0 = creating BPMs, 1 = reading directory
						bpm_multi_mode:				  0,	$	; 0 = single BPM, 1 = multiple BPMs
						;
						bpm_radio_flags:	  intarr(5),	$	; status of the five radio buttons on the page
						bpm_histo_flags:	  intarr(2),	$	; status of the two rbs
						;
						bpm_default_low_cut:	-90.0,		$
						bpm_default_high_cut:	+120.0,		$
						bpm_default_low_plot:	-150.0,		$
						bpm_default_high_plot:	+180.0,		$
						;
						bpm_low_cut:              -90.0,	$	; buffer for the text window
						bpm_high_cut:		      +120.0,	$
						;
						LED_scale:				    1.0,	$
						this_LED:				      0,	$
						;
						xx:						     0L,	$	; cursor x position
						yy:							 0L,	$	; cursor y position
						position:    		  [0,0,0,0], 	$   ; (x0,y0),(x1,y1) device corners of plot-data region
						bpm_histogram_x:	fltarr(513),	$	; the current histogram X vector
						bpm_histogram_y:	fltarr(513),	$
						bpm_histogram_n_low:	     0L,	$	; number of pixels falling off low histogram limit
						bpm_histogram_n_high:		 0L,	$	; same for high limit
						;
						this_BPM_no:			     0L,	$	; the current BPM structure index
						BPM:	REPLICATE(this_mask, bpm_max)	$	; the array of BPM mask info structures
					}
	;
	;	structure for linearity calc
	;
	Lin_struct = 	{	xx:						225L,		$	pixel coord to plot
						yy:						225L,		$
						;
						str:						'',		$
						BPM_path:					'',		$ 	path to the bad pixel file
						;
						LED_scale:				   1.0,		$
						this_LED:					 0,		$
						;
						position:			[0,0,0,0],		$
						;
						this_lin:					0L,		$
						this_lin_directory:			'',		$
						;
						no_lin:						 0,		$
						;
						l_groups:					0,		$		numbers of groups in droplist
						group_list:		    intarr(150),	$		pointers to group numbers in night list
						group_type:			strarr(150),	$		group types
						;
						b_groups:					0,		$		same for bias
						group_bias:			intarr(150),	$		pointers to bias groups
						;
						method:						0L,		$		; 1 or 2
						;
						;	method 1 selected linearity, bias groups
						;
						m1_lin_grp:					0L,		$
						m1_bias_grp:				0L,		$
						;
						;	method 2 selected lin, bias, 1st, last groups
						;
						m2_lin_first_grp:			0L,		$
						m2_lin_last_grp:			0L,		$
						m2_bias_first_grp:			0L,		$
						m2_bias_last_grp:			0L,		$
						;
						m2_n_lin_grps:				0L,		$
						m2_lin_grps:		intarr(200),	$
						m2_n_bias_grps:				0L,		$
						m2_bias_grps:		intarr(200),	$		; the group number of this bias group
						;
						m2_bias_n_paths:	intarr(200),	$		; number of images to coadd in this group
						m2_bias_ut_times:	dblarr(200),	$		; average UT of this group
						m2_bias_CDS_pathnames:	strarr(200,100),$	; pathnames to the CDS images
						m2_bias_RAW1_pathnames:	strarr(200,100),$	; pathnames to the RAW1 images
						;
						;	coadded biases
						;
						m2_bias_n_coadd:			0L,		$
						m2_bias_coadd_pathnames:	strarr(200),$
						m2_bias_coadd_JD:			dblarr(200),$
						;
						;	method 2 exposure time list and associated images
						;
						m2_n_exp_times:				0L,		$	;	number of different exposure times
						m2_exp_times:		fltarr(200),	$	; the different exposure times
						m2_exp_n_paths:		intarr(200),	$	; number of pathnames for each exp step
						m2_exp_CDS_pathnames:	strarr(200,100), $	; indexed by exp number, then list
						m2_exp_RAW1_pathnames:	strarr(200,100),	$
						;
						lin_id:				intarr(90),		$
						lin_UT:				dblarr(90),		$
						lin_night:			lonarr(90),		$
						lin_format:			strarr(90),		$
						lin_filter:			strarr(90),		$
						lin_directory:		strarr(90), 	$
						;
						;	space for lists after coadding
						;
						n_coadd:					0L,		$		not counting bias
						coadd_path_list: strarr(201),		$
						coadd_path_bias:			'',		$
						;
						;	space for image coadding
						;
						lin_data_ptr_0:		PTR_NEW(),		$		pointers to big data arrays of coadded images
						lin_data_ptr_1:		PTR_NEW(),		$
						lin_data_ptr_2:		PTR_NEW(),		$
						lin_data_ptr_3:		PTR_NEW(),		$
						;
						n_bias_list:			   0L,		$
						bias_list:		lonarr(1000),		$
						n_lin_list:				   0L,		$
						lin_list:		lonarr(1000),		$		image numbers for images in linearity list
						;
						n_bias_paths:				0L,		$
						bias_paths:		strarr(200),		$
						;
						n_lin_paths:	lonarr(200),		$
						n_lin_expt:					0L,		$
						lin_expt:		dblarr(200),		$		integration time for each time step
						lin_paths:		strarr(200,32),		$		up to 200 time steps, each with 32 images
						;
						;	the list of registered linearity tables  and pointer to 3 d linearity array in memory
						;
						lin_table_ptr:  	 PTR_NEW(),		$		pointer to the linearity tables
						n_lin_tables:			0L,			$		number of tables registered
						lin_table_0_path:	strarr(10)		$		pathname to the lin_0 file for up to 10 tables
					}
	;
	;	Ave RAW1 structure
	;
	a_group = 	{ 		serial_no:					0L,			$ 		this group number
						night_desig:				0L,			$		which night
						group_id:					0L,			$		which group in that night
						type:						'',			$		type of group
						coadds:						1,			$		number of coadds in each image in group
						ut:						0.0d0,			$		UT of first image in group
						object:						'',			$
						jd:						0.0d0,			$		Julian date (YYMMDDHHMMSS)
						reg_lin_no:					0L,			$		registered linearity table number to use for correction
						output_RAW1_name:			'',			$		filename (not pathname) of output ave RAW1
						output_FITS_name:			'',			$		filename (not pathname) of output FITS image
						n_images:					0L,			$		number of images in the pathname lists below
						image_CDS_pathname:	strarr(1000),		$		CDS full pathnames to the input images
						image_RAW1_pathname: strarr(1000)		$		same, for RAW1 pathnames
				}
	;
	Ave_RAW1_struct = {	n_groups:					0L,			$		number of groups
						this_group_no:				0L,			$		group numbers of bias groups
						output_directory:			'',			$		directory path for output images
						;
						group:	REPLICATE(a_group,5000)			$		sub_structure for each group

						}
	;
	;	linearity correct structures
	;
	lc_struct = 		{ n_groups:					0L,			$
						  this_group_no:			0L,			$
						  view_flag:				0L,			$		0 = not yet edited, 1 = edited
						  output_directory:			'',			$		directory path, relative to root_reduced
						  group:  REPLICATE(a_group,250)		$
						}
	;
	;	dark coadding structure
	;
	dark_struct = 		{ n_nights:					0L,			$		number of nights with possible darks
						  night_list:		lonarr(50),			$		list of nights with possible darks
						  ;
						  n_dark_groups:			0L,			$
						  night_desig:		lonarr(300),		$		night desig for each group
						  night_group_id:	lonarr(300),		$		group id of each group in its night
						  exptime:			fltarr(300),		$		exposure time (nominal CDS)
						  n_input_pathnames:lonarr(300),		$		number of pathnames for each group
						  input_pathname:	strarr(300,128),	$		full pathnames to LC input images
						  input_first_JD:	dblarr(300),		$		JD of first image in each group
						  output_pathname:	strarr(300),		$		output pathname for each coadded dark image
						  ;
						  view_flag:				0L,			$
						  data_array_ptr:	PTR_NEW()			$		pointer to the 3d array of input image data
						  }
	;
	;	flat-field structure
	;
	polar_save = 		{ group:			lonarr(3000),		$
						  image_no:			lonarr(3000),		$
						  HWP:				dblarr(3000),		$
						  camera:			strarr(3000),		$
						  lites:			strarr(3000),		$
						  filter:			strarr(3000),		$
						  JD:				dblarr(3000),		$
						  pathname:			strarr(3000),		$
						  n_images:			           0L		$
						}
	;
	polar_group = 		{ night_desig:				0L,			$
						  HWP:					   0.0d0,		$
						  lites:					'',			$
						  camera:					'',			$
						  filter:					'',			$
						  JD:						0.0d0,		$
						  output_pathname:			'',			$
						  n_pathnames:				0L,			$
						  pathnames:		strarr(200)			$
						 }
	;
	polar_flat = 		{ polar_group: REPLICATE(polar_group, 33),$
						  n_polar_groups:			0L			$
						}
	;
	ff_struct = 		{ n_nights:					0L,			$
						  night_list:		lonarr(50),			$
						  ;
						  n_flat_groups:			0L,			$
						  night_desig:		lonarr(300),		$		night desig for each group
						  night_group_id:	lonarr(300),		$		group id of each group in its night
						  exptime:			fltarr(300),		$		exposure time (nominal CDS)
						  n_input_pathnames:lonarr(300),		$		number of pathnames for each group
						  input_pathname:	strarr(300,128),	$		full pathnames to LC input images
						  input_first_JD:	dblarr(300),		$		JD of first image in each group
						  output_pathname:	strarr(300),		$		output pathname for each coadded dark image
						  group_type:		strarr(300),		$
						  group_filter:		strarr(300),		$
						  group_camera:		strarr(300),		$
						  HWP_angle:		fltarr(300),		$
						  ;
						  ;	polar FF variables
						  ;
						  polar_save:		polar_save,			$		collection of raw info for each image
						  ;
						  polar_flat:	REPLICATE(polar_flat, 10), $	up to 10 combinations of polar flat sets
						  n_polar_flats:				0L,			$
						  ;
						  ;	zero hop problem variables
						  ;
						  n_pol_groups:				0L,			$		number of distinct polar groups present in directory
						  mean_PA:			fltarr(10),			$		mean PA for this group
						  pol_group_ymd:	strarr(10),			$		names of each group as YYYYMMDD
						  pol_group_filter:	strarr(10),			$		filter for this group
						  zero_hop_fix_status: intarr(10),		$		status of polar groups - 0 = no group
						  											;		1 = group doesn't need fixing
						  											;		2 = group needs fixing
						  ;
						  view_flag:				0L,			$
						  ;data_array_ptr:	PTR_NEW(),			$
						  ;
						  fm_ptr:			PTR_NEW(),			$		; place holder for pointer
						  match:		lonarr(2,300),			$		; indices back to group info above 0=On, 1=Off
						  n_match:					0L,			$
						  n_unmatch:				0L,			$
						  difference_pathname: strarr(300),		$
						  ;
						  ;	for different modes: 0=pol, 1=imaging, 2=JHKspec, 3=Kspec, 4=Jspec
						  ;
;						  col_range:	[[100,900], [100,900], [100,900], $
;						  				 [100,900], [100,900]],	$		; min, max cols for histo calc
;						  row_range:	[[300,700], [300,700], [300,700], $
;						  				 [300,700], [300,700]],		$		;	same for rows
						  ;
						  normal:			dblarr(300)		$
						  }
	;
	;	read time structure
	;
	read_time_struct = 	{	n_nights:				0L,			$
						  	night_list:		lonarr(50),			$
						  ;
						  	lin_dir:				'',			$
							n_list:					0L,			$
							path_list:		strarr(15),			$
							status:			strarr(15),			$
							;
							data_ptr:		PTR_NEW(),			$
							exptime:		dblarr(15),			$
							path_0:					'',			$
							outpath:				'',			$
							;
							read_time_value:	0.223114d0		$
						}
	;
	;	bad pixel, dark, flat correction structure
	;
	bad_dark_flat_struct = {	n_nights:				0L,			$
								night_list:		lonarr(50),			$
								this_night_number:		0L,			$
								;
								n_groups:				0L,			$
								group_index:	lonarr(600),		$
								group_bpm_index:	lonarr(600),	$
								group_dark_index:	lonarr(600),	$
								group_flat_index:	lonarr(600),	$
								group_polar_flat_index:	lonarr(600),$
								status:				strarr(600),	$
								this_group_number:		0L,			$
								n_fixed_groups:			0L,			$
								;
								n_images:				0L,			$
								image_list:		strarr(4000),		$
								this_image_number:		0L,			$
								this_image_ut:			0.0d0,		$
								this_image_jd:			0.0d0,		$
								;
								;------------------------------------
								n_bpm:					0L,			$
								bpm_path:		strarr(1000),		$
								bpm_ut:			dblarr(1000),		$
								bpm_jd:			dblarr(1000),		$
								;
								this_bpm_number:		0L,			$
								bpm_image_ptr:		PTR_NEW(),		$
								bpm_header_ptr:		PTR_NEW(),		$
								;------------------------------------
								n_dark:					0L,			$
								dark_path:		strarr(1000),		$
								dark_ut:		dblarr(1000),		$
								dark_jd:		dblarr(1000),		$
								dark_exptime:	fltarr(1000),		$
								;
								this_dark_number:		0L,			$
								dark_image_ptr:		PTR_NEW(),		$
								dark_header_ptr:	PTR_NEW(),		$
								;------------------------------------
								n_flat:					0L,			$
								flat_path:		strarr(350),		$
								flat_jd:		dblarr(350),		$
								flat_ut:		dblarr(350),		$
								flat_type:		strarr(350),		$
								flat_camera:	strarr(350),		$
								flat_exptime:	fltarr(350),		$
								flat_filter:	strarr(350),		$
								;
								this_flat_number:		0L,			$
								flat_image_ptr:		PTR_NEW(),		$
								flat_header_ptr:	PTR_NEW(),		$
								;------------------------------------
								n_polar_flat_sets:			0L,		$	number of distinct polar flat sets
								polar_flat_mean_PA:  fltarr(10),	$	mean PA for this polar flat set (for Zero Hop)
								polar_flat_set_elements: intarr(10),$	number of consecutive polar flats in each set
								polar_flat_set_filter: strarr(10),	$	filter name for each polar flat set
								polar_flat_set_jd:	dblarr(10),		$	starting jd for each polar flat set
								;
								n_polar_flat:				0L,		$
								polar_flat_path:	strarr(350),	$
								polar_flat_jd:		dblarr(350),	$
								polar_flat_ut:		dblarr(350),	$
								polar_flat_type:	strarr(350),	$
								polar_flat_camera:	strarr(350),	$
								polar_flat_HWP:		dblarr(350),	$
								polar_flat_filter:	strarr(350),	$
								;
								this_polar_flat_number:		0L,		$
								polar_flat_image_ptr:	PTR_NEW(),	$
								polar_flat_header_ptr:	PTR_NEW()	$
								;------------------------------------
							}
	;
    ;
    ;	observation type list
    ;
	;    obs_types = [ "Dark", "Bias", "P-Flat-On", "P-Flat-Off", "S-Flat-On", "S-Flat-Off",$
	;                  "I-Flat-On", "I-Flat-Off", "Lin-On", "Lin-Off", "Pol-Spec", "Pol-Img",$
	;                  "Spec", "S-Argon", "Image" , "R-Flat-On", "R-Flat-Off", "PCal-On", "PCal-Off"]
    ;
    ;-------------------------------------------------------------------
	;
	; structure for filter name matching
	;
	type = {FW: strarr(4,3), Name: '', Camera: 0}
	;
	i_types = 33
	types = REPLICATE(type, i_types)
	;
	;	the order is FW1, FW2, FW3, Pol wheel
	;
	;	names are: (as of 20100729)
	;
	;	FW1: Dark, J, H, Ks, Lprime, Mprime, IZYJ-BP, 1.40LP, PK50-2, Open
	;   FW2: Dark, SED_X-Grism, 1.17LP, 1.85LP, PK501, IZYJ-Blocker, H2-On, H2-Off, Open
	;   FW3: Dark, JHK_Grism, Absent, Absent, Absent, Absent, Open, LM_Grism, WireGrid, OPen
	;	POL: Dark, F/5_Baffle, F/17_Baffle, H_HWP, K_HWP, Open
	;
	;	Camera: 0 = any; 5 = F/5 only; 17 = F/17 only
	;
	;	modes (by number and name) are:
	;
	;	mode 1 - Dark
	types[0].Name = 'Dark'
	types[0].FW = [['Dark', 	'*',		'*',		'*'],$
					['Dark','Open','OPen','Open'],$
				    ['Dark','Open','Open','Open']]
	types[0].Camera = 0
	;
	;	mode 2 - All Dark
	types[1].Name = 'All_Dark'
	types[1].FW = [['Dark', 	'Dark',		'Dark',		'Dark'],$
					['','','',''],$
				    ['','','','']]
	types[1].Camera = 0
	;
	;	mode 3 - J-Img
	types[2].Name = 'J-Img'
	types[2].FW = [['J', 	'PK501',		'Absent',		'Open'], $
				   ['J', 	'PK501',		'Open',		'Open'],$		; alternate descriptions
				   ['J','PK50_1','Open','Open']]
	types[2].Camera = 0
	;
	;	mode 4 - H-Img
	types[3].Name = 'H-Img'
	types[3].FW = [['H', 	'PK501',		'Absent',		'Open'],$
			       ['H', 	'PK501',		'Open',		'Open'],$
			       ['H','PK50_1','Open','Open']]
	types[3].Camera = 0
	;
	;	mode 5 - K-Img
	types[4].Name = 'K-Img'
	types[4].FW = [['Ks1', 	'PK501',		'Absent',		'Open'],$
				   ['Ks1', 	'PK501',		'Open',		'Open'],$
				   ['Ks','PK50_1','Open','Open']]
	types[4].Camera = 0
	;
	;	mode 6 - L-Img
	types[5].Name = 'L-Img'
	types[5].FW = [['Lprime', 	'Open',		'Absent',		'f17_baffle'],$
				   ['Lprime', 	'Open',		'Open',		'Open'],$
				   ['Lprime', 	'Open',		'Open',		'Open']]
	types[5].Camera = 0
	;
	;	mode 7 - M-Img
	types[6].Name = 'M-Img'
	types[6].FW = [['Mprime', 	'Open',		'Absent',		'f17_baffle'],$
				   ['Mprime', 	'Open',		'Open',		'f17_baffle'],$
				   ['Mprime', 	'Open',		'Open',		'Open']]
	types[6].Camera = 0
	;
	;	mode 8 - H2On-Img
	types[7].Name = 'H2On-Img'
	types[7].FW = [['H2On', 	'PK501',		'Absent',		'Open'],$
				   ['H2On', 	'PK501',		'Open',		'Open'],$
				   ['PK50-2','H2-On','Open','Open']]
	types[7].Camera = 0
	;
	;	mode 9 - H2Off-Img
	types[8].Name = 'H2Off-Img'
	types[8].FW = [['H2Off', 	'PK501',		'Absent',		'Open'],$
				   ['H2Off', 	'PK501',		'Open',		'Open'],$
				   ['PK50-2','H2-Off','Open','Open']]
	types[8].Camera = 0
	;
	;	mode 10 - IZYJ-Wide-Img
	types[9].Name = 'IZYJ-W-Img'
	types[9].FW = [['PK50-2', 	'IZYJ-Blocker',		'Open',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[9].Camera = 0
	;
	;	mode 11 - IZYJ-Med-Img
	types[10].Name = 'IZYJ-M-Img'
	types[10].FW = [['IZYJ-BP', 	'PK50_1',		'Open',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[10].Camera = 0
	;
	;	mode 12 - H-Pol
	types[11].Name = 'H-Pol'
	types[11].FW = [['H',		'PK501',		'WireGrid',		'H_HWP'],$
				   ['H','PK50_1','WireGrid','H_HWP'],$
				   ['','','','']]
	types[11].Camera = 0
	;
	;	mode 13 - K-Pol
	types[12].Name = 'K-Pol'
	types[12].FW = [['Ks',		'PK50_1',		'WireGrid',		'K_HWP'],$
				   ['','','',''],$
				   ['','','','']]
	types[12].Camera = 0
	;
	;	mode 14 - H2On-Pol
	types[13].Name = 'H2On-Pol'
	types[13].FW = [['PK50-2',		'H2-On',		'WireGrid',		'K_HWP'],$
				   ['','','',''],$
				   ['','','','']]
	types[13].Camera = 0
	;
	;	mode 15 - H2Off-Pol
	types[14].Name = 'H2Off-Pol'
	types[14].FW = [['PK50-2',		'H2-Off',		'WireGrid',		'K_HWP'],$
				   ['','','',''],$
				   ['','','','']]
	types[14].Camera = 0
	;
	;	mode 16 - JHK-Spec
	types[15].Name = 'JHK-Spec'
	types[15].FW = [['Open', 	'1.17LP',		'JHK_Grism',		'Open'],$
				   ['PK50_2','1.17LP','JHK_Grism','Open'],$
				   ['Open','1.17LP','JHK_Grism','Open']]
	types[15].Camera = 5
	;
	;	mode 17 - HK-Spec
	types[16].Name = 'HK-Spec'
	types[16].FW = [['Open', 	'1.40LP_1',		'JHK_Grism',		'Open'],$
				   ['PK502', 	'1.40LP_1',		'JHK_Grism',		'Open'],$
				   ['1.40LP','Open','JHK_Grism','Open']]
	types[16].Camera = 5
	;
	;	mode 18 - H17-Spec
	types[17].Name = 'H17-Spec'
	types[17].FW = [['1.40LP', 	'Open',		'JHK_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[17].Camera = 17
	;
	;	mode 19 - IZYJ-Spec
	types[18].Name = 'IZYJ-Spec'
	types[18].FW = [['IZYJ-BP', 	'IZYJ-Blocker',		'JHK_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[18].Camera = 5
	;
	;	mode 20 - IZYJ-BP-Spec
	types[19].Name = 'IZYJ-BP-Spec'
	types[19].FW = [['IZYJ-BP', 	'PK50_1',		'JHK_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[19].Camera = 5
	;
	;	mode 21 - IZYJ-BKR-Spec
	types[20].Name = 'IZYJ-BKR-Spec'
	types[20].FW = [['PK50-2', 	'IZYJ-Blocker',		'JHK_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[20].Camera = 5
	;
	;	mode 22 - IZ17-Spec
	types[21].Name = 'IZ17-Spec'
	types[21].FW = [['PK50-2', 	'IZYJ-Blocker',		'JHK_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[21].Camera = 17
	;
	;	mode 23 - IZY17-Spec --- same mode, but wants camera offset
	types[22].Name = 'IZY17-Spec'
	types[22].FW = [['PK50-2', 	'IZYJ-Blocker',		'JHK_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[22].Camera = 17
	;
	;	mode 24 - LM-Spec
	types[23].Name = 'LM-Spec'
	types[23].FW = [['Open', 	'2.8LP',		'LM_Grism',		'f17_baffle'],$
					['Open', 	'2.8LP',		'LM_Grism',		'Open'],$
					['Open','2.8LP','LM_Grism','Open']]
	types[23].Camera = 5
	;
	;	mode 25 - L-Spec
	types[24].Name = 'L-Spec'
	types[24].FW = [['Lprime', 	'Absent',		'LM_Grism',		'f17_baffle'],$
					['Lprime', 	'Open',			'LM_Grism',		'f17_baffle'],$
					['Lprime', 	'Open',			'LM_Grism',		'Open']]
	types[24].Camera = 5
	;
	;	mode 26 - M-Spec
	types[25].Name = 'M-Spec'
	types[25].FW = [['Mprime', 	'Absent',		'LM_Grism',		'f17_baffle'],$
				    ['Mprime', 	'Open'	,		'LM_Grism',		'f17_baffle'],$
				    ['Mprime', 	'Open',			'LM_Grism',		'Open']]
	;
	;	mode 27 - K17-Spec
	types[26].Name = 'K17-Spec'
	types[26].FW = [['PK502', 	'1.85LP',		'LM_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['PK50-2','1.85LP','LM_Grism','Open']]
	types[26].Camera = 17
	;
	;
	;	mode 28 - LM17-Spec
	types[27].Name = 'LM17-Spec'
	types[27].FW = [['Open', 	'2.8LP',		'LM_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[27].Camera = 17
	;
	;	mode 29 - L17-Spec
	types[28].Name = 'L17-Spec'
	types[28].FW = [['Lprime', 	'Open',		'LM_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[28].Camera = 17
	;
	;	mode 30 - M17-Spec
	types[29].Name = 'M17-Spec'
	types[29].FW = [['Mprime', 	'Open',		'LM_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[29].Camera = 17
	;
	;	mode 31 - J-Spec
	types[30].Name = 'J-Spec'
	types[30].FW = [['J', 	'PK50_1',		'LM_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[30].Camera = 5
	;
	;	mode 32 - J17-Spec
	types[31].Name = 'J17-Spec'
	types[31].FW = [['J', 	'PK50_1',		'LM_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[30].Camera = 17
	;
	;	mode 33 - IZYJ-X-Spec
	types[32].Name = 'IZYJ-X-Spec'
	types[32].FW = [['IZYJ-BP', 	'SED_X-Grism',		'JHK_Grism',		'Open'],$
				   ['', 	'',		'',		''],$
				   ['','','','']]
	types[32].Camera = 5

	;
	filt_struct = {types: types}
	;
	;------------------------------------------------------------------------
    ;
    PTR_summary = PTR_NEW(summary_struct,/NO_COPY)
    PTR_night = PTR_NEW(night_struct,/NO_COPY)
    PTR_FITS = PTR_NEW(FITS_struct,/NO_COPY)
    PTR_filt = PTR_NEW(filt_struct,/NO_COPY)
    PTR_kokopeli = PTR_NEW(k_struct,/NO_COPY)
    PTR_RAW1 = PTR_NEW(RAW1_struct,/NO_COPY)
    PTR_BPM = PTR_NEW(BPM_struct,/NO_COPY)
    PTR_Linearity = PTR_NEW(Lin_struct,/NO_COPY)
    PTR_Ave_RAW1 = PTR_NEW(Ave_RAW1_struct,/NO_COPY)
    PTR_LC = PTR_NEW(lc_struct,/NO_COPY)
    PTR_Dark = PTR_NEW(dark_struct,/NO_COPY)
    PTR_Flat = PTR_NEW(ff_struct,/NO_COPY)
    PTR_RTime = PTR_NEW(read_time_struct,/NO_COPY)
    PTR_BDF = PTR_NEW(bad_dark_flat_struct,/NO_COPY)
    PTR_Column = PTR_NEW(c_struct,/NO_COPY)
    ;
    ;	load the pointer(s) into the pointer structure
    ;
    PTR_struct = {PTR_summary: PTR_summary, PTR_night: PTR_night, PTR_FITS: PTR_FITS, $
    				PTR_filt: PTR_filt, PTR_kokopeli: PTR_kokopeli, PTR_RAW1: PTR_RAW1, $
    				PTR_BPM: PTR_BPM, PTR_Linearity: PTR_Linearity, PTR_Ave_RAW1: PTR_Ave_RAW1,$
    				PTR_LC: PTR_LC, PTR_Dark: PTR_Dark, PTR_Flat: PTR_Flat, $
    				PTR_RTime: PTR_RTime, PTR_BDF: PTR_BDF, PTR_Column: PTR_Column}		; add more as needed (night, group, etc)
    ;
    WIDGET_CONTROL, DEFAULT_FONT=(*PTR_summary).default_font
    ;
    WIDGET_CONTROL, wWidget, SET_UVALUE=PTR_struct
end
;
;-----------------------------------------------------------------
pro MSP_BDP_Realize, wWidget
	;
	;	get key window information
	;
	WIDGET_CONTROL, wWidget, GET_UVALUE=PTR_struct
	;
	s_ptr = PTR_struct.PTR_summary
	;
	(*s_ptr).echo_window_wid = WIDGET_INFO( wWidget, FIND_BY_UNAME="Base_Bottom_Text_Window")
	;
	;	need to pass an event structure:
	;
	event = {top: wWidget}
	;
	TLB_Title = (*s_ptr).version_string + $
    			' '+' Unknown '+'  '+SYSTIME()+' for Run '+'TBD '
    tlb_wid = WIDGET_INFO( Event.top, FIND_BY_UNAME='MSP_BDP_Base')
    ;
    WIDGET_CONTROL, tlb_wid, TLB_SET_TITLE=TLB_Title
	;
	; desensitize P1, P2 widget buttons below first two
	;
	P1_get_wids, event
	P2_get_wids, event
	P2_get_LED_wids, event
	P3_get_wids, event
	P3_get_LED_wids, event
	P4_get_wids, event
	P5_get_wids, event
	P6_get_wids, event
	P7_get_wids, event
	P8_get_wids, event
	P9_get_wids, event
	P10_get_wids, event
	P11_get_wids, event
	P12_get_wids, event
	;
	sense = 1
	P1_button_number = 0
	P1_button_sensitize, Event, P1_button_number, sense
	P1_button_number = 1
	P1_button_sensitize, Event, P1_button_number, sense
	P1_button_sensitize, Event, 7, sense
	;
	;	point to the first tab (summary) and write the summary to the screen
	;
	tab_wid = WIDGET_INFO(wWidget, FIND_BY_UNAME="Big_Tab")
	WIDGET_CONTROL, tab_wid, SET_TAB_CURRENT=0
	;
	;	capture operating system family and set tabbing multiline variable based on family
	;
	os_family = STRUPCASE(!VERSION.OS_FAMILY)
	if(os_family eq 'WINDOWS') then begin
		multiline = 0
	endif else begin
		multiline = 4
	endelse
	WIDGET_CONTROL, tab_wid, SET_TAB_MULTILINE=multiline
	;
	Summary_print, wWidget
	;
	MSP_BDP_print_text, event, "Mimir Software Package: Basic Data Processor - Initialized!"
	;
end
;
;------------------------------------- Print Summary ----------------
pro Summary_print, wWidget
    ;
    ;  write text to the summary window
    ;
    summary_ID = WIDGET_INFO( wWidget, FIND_BY_UNAME="Summary_Text_Window")
    ;file_in = dialog_pickfile()
    ;print, "File name = "
    file_in = 'MSP_BDP_Summary.txt'
    ;
    ;print, file_in
    openr, lu_in, file_in, /GET_LUN
    ist = 0
    WHILE ~EOF(lu_in) do begin
       line = ''
       readf, lu_in, line
       WIDGET_CONTROL, summary_ID, SET_VALUE=line, /APPEND
       ist++
    ENDWHILE
    FREE_LUN, lu_in
    ;print, "read in ",ist," lines"
end
;-----------------------------------------------------------------
pro Exit_Asserted, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct	;	close log file
	s_ptr = PTR_struct.PTR_summary
	f_ptr = PTR_struct.PTR_FITS
	n_ptr = PTR_struct.PTR_night
	t_ptr = PTR_struct.PTR_filt
	k_ptr = PTR_struct.PTR_kokopeli
	r_ptr = PTR_struct.PTR_RAW1
	b_ptr = PTR_struct.PTR_BPM
	l_ptr = PTR_struct.PTR_Linearity
	a_ptr = PTR_struct.PTR_Ave_RAW1
	lc_ptr = PTR_struct.PTR_LC
	d_ptr = PTR_struct.PTR_Dark
	ff_ptr = PTR_struct.PTR_Flat
	rt_ptr = PTR_struct.PTR_RTime
	bdf_ptr = PTR_struct.PTR_BDF
	c_ptr = PTR_struct.PTR_Column
	;
	if( (*s_ptr).lu_log ne 0) then FREE_LUN, (*s_ptr).lu_log
	;
	;free the pointers
	;
	;		first the ones contained in structures
	;
	PTR_FREE, (*d_ptr).data_array_ptr
	PTR_FREE, (*l_ptr).lin_data_ptr_0
	PTR_FREE, (*l_ptr).lin_data_ptr_1
	PTR_FREE, (*l_ptr).lin_data_ptr_2
	PTR_FREE, (*l_ptr).lin_data_ptr_3
	PTR_FREE, (*l_ptr).lin_table_ptr
	PTR_FREE, (*rt_ptr).data_ptr
	;
	PTR_FREE, (*bdf_ptr).bpm_image_ptr
	PTR_FREE, (*bdf_ptr).bpm_header_ptr
	PTR_FREE, (*bdf_ptr).dark_image_ptr
	PTR_FREE, (*bdf_ptr).dark_header_ptr
	PTR_FREE, (*bdf_ptr).flat_image_ptr
	PTR_FREE, (*bdf_ptr).flat_header_ptr
	;
	;	and then the full structure pointers
	;
	PTR_FREE, s_ptr	;
	PTR_FREE, f_ptr
	PTR_FREE, n_ptr
	PTR_FREE, t_ptr
	PTR_FREE, k_ptr
	PTR_FREE, r_ptr
	PTR_FREE, b_ptr
	PTR_FREE, l_ptr
	PTR_FREE, a_ptr
	PTR_FREE, lc_ptr
	PTR_FREE, d_ptr
	PTR_FREE, ff_ptr
	PTR_FREE, rt_ptr
	PTR_FREE, bdf_ptr
	PTR_FREE, c_ptr
	;
	WIDGET_CONTROL, Event.top, /DESTROY
end
;
;----------------------------- Part 1 Event Handler
pro P1_Handler, Event
	;
	;	figure out which button was pushed
	;
	buttons = [	"P1_Button_Create_New_Run", $
				"P1_Button_Open_Existing_Run",$
				"P1_Button_Reconnect",$
				"P1_Button_Show_Night_Summary",$
				"P1_Button_Add_Nights",$
				"P1_Button_Delete_Nights",$
				"P1_Button_Save_Night_Summary",$
				"P1_Button_Discover_Raw_Nights",$
				"P1_Button_Print_Night_Summary" $
			   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P1_Create_New_Run_Name, Event
			button_wids[1]: P1_Open_Existing_Run, Event
			button_wids[2]: P1_Reconnect_Directories, Event
			button_wids[3]: P1_Show_Night_Summary, Event
			button_wids[4]: P1_Add_Nights, Event
			button_wids[5]: P1_Delete_Nights, Event
			button_wids[6]: P1_Write_Summary_File, Event
			button_wids[7]: P1_Discover_Raw_Nights, Event
			button_wids[8]: P1_Print_Run_Summary_File, Event
			else: MSP_BDP_print_text, Event, "Error - P1 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;----------------------------- Part 2 Event Handler
pro P2_Handler, Event
	;
	;	figure out which button was pushed
	;
	buttons = [	"P2_Button_Build_Image_Log",$
				"P2_Button_Edit_Image_Table", $
				"P2_Button_Group_Images",$
				"P2_Droplist_Select_Night",$
				"P2_Button_Perform_Directory_Check",$
				"P2_Button_Edit_Group_Table",$
				"P2_Button_Read_Directory_for_Image_List",$
				"P2_Button_Save_Image_Group_Tables",$
				"P2_Button_Open_Existing_Tables", $
				"P2_Button_Print_Group_Table", $
				"P2_Button_Print_Image_Table", $
				"P2_Button_Run_Pipeline" $
			   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P2_Build_Image_Log, Event
			button_wids[1]: P2_Edit_Image_Table, Event
			button_wids[2]: P2_Group_Images, Event
			button_wids[3]: P2_Select_Night, Event
			button_wids[4]: P2_Perform_Directory_Check, Event
			button_wids[5]: P2_Edit_Group_Table, Event
			button_wids[6]: P2_Read_Directory_for_Image_List, Event
			button_wids[7]: P2_Save_Image_Group_Tables, Event
			button_wids[8]: P2_Open_Existing_Night_Summaries, Event
			button_wids[9]: P2_Print_Group_Table, Event
			button_wids[10]: P2_Print_Image_Table, Event
			button_wids[11]: P2_Run_Pipeline, Event
			else:	MSP_BDP_print_text, Event, "Error - P2 button push not recognized"
	    endcase
	endif else 	MSP_BDP_print_text, Event, "No button pushed."
end
;
;----------------------------- Part 3 Event Handler
pro P3_Handler, Event
	;
	buttons = [	"P3_Button_Start_Kokopeli",$
				"P3_Droplist_Select_Night",$
				"P3_Button_View_Edit_Shifts",$
				"P3_Button_Zero_Shifts",$
				"P3_Button_Ignore_Shifts",$
				"P3_Button_Do_Shifts",$
				"P3_Column_Test_Checkbox", $
				"P3_Button_Run_Pipeline" $
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P3_Start_Kokopeli, Event
			button_wids[1]: P3_Select_Night, Event
			button_wids[2]: P3_View_Edit_Shifts, Event
			button_wids[3]: P3_Zero_Shifts, Event
			button_wids[4]: P3_Ignore_Shifts, Event
			button_wids[5]: P3_Do_Shifts, Event
			button_wids[6]: P3_Toggle_Checkbox, Event
			button_wids[7]: P3_Run_Pipeline, Event
			else:	MSP_BDP_print_text, Event, "Error - P3 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;----------------------------- Part 4 Event Handler
pro P4_Handler, Event
	;
	buttons = [	"P4_Button_Start_RAW1_Examination",$
				"P4_Button_Print_Summary",$
				"P4_Button_Zoom",$
				"P4_Button_Unzoom",$
				"P4_Button_Mark_Start",$
				"P4_Button_Mark_End",$
				"P4_Button_Plot_Start_to_End",$
				"P4_Button_Show_Groups",$
				"P4_Button_Process_Jumps",$
				"P4_Button_Clear_Jumps",$
				"P4_Button_Mark_Jump", $
				"P4_Button_Done",$
				"P4_Button_Correct_Jumps", $
				;
				"P4_Droplist_Select_Night"$
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P4_RAW1_Examination, Event
			button_wids[1]: P4_Print_Summary, Event
			button_wids[2]: P4_Modify_Plot, Event, "Z"
			button_wids[3]: P4_Modify_Plot, Event, "U"
			button_wids[4]: P4_Modify_Plot, Event, "S"
			button_wids[5]: P4_Modify_Plot, Event, "E"
			button_wids[6]: P4_Modify_plot, Event, "P"
			button_wids[7]: P4_Modify_plot, Event, "G"
			button_wids[8]: P4_Modify_plot, Event, "F"
			button_wids[9]: P4_Modify_plot, Event, "C"
			button_wids[10]: P4_Modify_Plot, Event, "J"
			button_wids[11]: P4_Done, Event
			button_wids[12]: P4_Correct_Jumps, Event
			;
			button_wids[13]: P4_Select_Night, Event
			else:	MSP_BDP_print_text, Event, "Error - P4 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;-----------------------------------------------------------------
pro P4_Keyboard_Handler, Event
	;
	;	which character?
	;
	ch = String(byte(Event.ch))
	press = Event.press
	if(press eq 1) then	begin
		;
		;	command list
		;
		CH = STRUPCASE(ch)
		commands = ["Z","U","S","E","P", "G", "F","D", "J", "C"]
		ind = WHERE( commands[*] eq CH, count)
		if(count ne 1) then goto, P4KH_end
		;
		;	legit command, process it
		;
		P4_Modify_Plot, Event, CH
	endif
	;
	P4KH_end:
end
;
;-----------------------------------------------------------------
pro P4_Mouse_Handler, Event
	;
	;	used to get the cursor position in the draw window
	;
	if( Event.press eq 1) then begin
		WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
		r_ptr = PTR_struct.PTR_RAW1
		;
		(*r_ptr).xx = Event.x
		(*r_ptr).yy = Event.y
		;
		;print, "x, y = ",Event.x, Event.y
	endif
end
;
;------------------------------- Part 5 (BPM) Handler
pro P5_Handler, Event
	;
	buttons = [	"P5_Droplist_Select_Night",$
				;
				"P5_Button_View_Edit_Group_BPM_Table",$
				;
				"P5_Button_Start_BPM_Creation",$
				;
				"P5_Radio_Button_One",	$
				"P5_Radio_Button_All",$
				;
				"P5_Radio_Button_Never",$
				"P5_Radio_Button_First",$
				"P5_Radio_Button_Every", $
				;
				"P5_Radio_Button_Single", $
				"P5_Radio_Button_Multiple", $
				;
				"P5_Button_Low_Cut",$
				"P5_Button_High_Cut",$
				"P5_Button_Continue",$
				;
				"P5_Droplist_Select_BPM_Histo" $
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P5_Select_Night, Event
			button_wids[1]: P5_Show_Table, Event
			button_wids[2]: P5_Start_BPM_Creation, Event
			button_wids[3]: P5_Radio_Button_0, Event, 0
			button_wids[4]: P5_Radio_Button_0, Event, 1
			button_wids[5]: P5_Radio_Button_1, Event, 0
			button_wids[6]: P5_Radio_Button_1, Event, 1
			button_wids[7]: P5_Radio_Button_1, Event, 2
			button_wids[8]: P5_Radio_Button_2, Event, 0
			button_wids[9]: P5_Radio_Button_2, Event, 1
			button_wids[10]: P5_Modify_Plot, Event, "L"
			button_wids[11]: P5_Modify_Plot, Event, "H"
			button_wids[12]: P5_Modify_Plot, Event, "C"
			button_wids[13]: P5_Select_BPM, Event
			;
			else:	MSP_BDP_print_text, Event, "Error - P5 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;-----------------------------------------------------------------
pro P5_Mouse_Handler, Event
	;
	;	used to get the cursor position in the draw window
	;
	if( Event.press eq 1) then begin
		WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
		b_ptr = PTR_struct.PTR_BPM
		;
		(*b_ptr).xx = Event.x
		(*b_ptr).yy = Event.y
		;
		;print, "x, y = ",Event.x, Event.y
	endif
end
;
;-----------------------------------------------------------------
pro P5_Keyboard_Handler, Event
	;
	;	which character?
	;
	ch = String(byte(Event.ch))
	press = Event.press
	if(press eq 1) then	begin
		;
		;	command list
		;
		CH = STRUPCASE(ch)
		commands = ["L","H", "C"]
		ind = WHERE( commands[*] eq CH, count)
		if(count ne 1) then goto, P5KH_end
		;
		;	legit command, process it
		;
		P5_Modify_Plot, Event, CH
	endif
	;
	P5KH_end:
end
;-----------------------------------------------------------------
pro P6_Handler, Event
	;
	buttons = [	"P6_Button_Make_New_Tables",$
				"P6_Button_List_All_Tables",$
				"P6_Button_Import_Existing_Table",$
				"P6_Droplist_Nights",$
				;
				;	Method 1 widgets
				;
				"P6_Droplist_Linearity_Groups",$
				"P6_Droplist_Bias_Groups",$
				;
				"P6_Button_Make_Matches",$
				"P6_Button_View_Matches",$
				"P6_Button_Combine_Matches",$
				;
				;	Method 2 widgets
				;
				"P6_Droplist_Linearity_First_Group",$
				"P6_Droplist_Bias_First_Group",$
				"P6_Droplist_Linearity_Last_Group",$
				"P6_Droplist_Bias_Last_Group",$
				;
				"P6_Button_Make_Matches_Method_2",$
				"P6_Button_View_Matches_Method_2",$
				"P6_Button_Combine_Matches_Method_2",$
				;
				;	Combined methods processing
				;
				"P6_Button_Read_Directory",$
				"P6_Button_View_Edit_Coadded_Images",$
				"P6_Button_Read_Coadded_Images",$
				"P6_Button_Show_Pixel_Plot",$
				;
				"P6_Button_Start_Linearity_Computation"$
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P6_Select_Make_New_Tables, Event
			button_wids[1]: P6_List_All_Tables, Event
			button_wids[2]: P6_Import_Existing_Table, Event
			button_wids[3]: P6_Select_Night, Event
			;
			;	method 1 - deprecated - DPC - 20101224
			;
			;button_wids[4]: P6_Select_Linearity_Group, Event, 1		; 1 = droplist number
			;button_wids[5]: P6_Select_Bias_Group, Event, 2
			;button_wids[6]: P6_Make_Image_Matches, Event
			;button_wids[7]: P6_View_Image_Match_Table, Event
			;button_wids[8]: P6_Combine_Image_Matches, Event
			;
			;	method 2
			;
			button_wids[9]: P6_Select_Linearity_Group, Event, 3
			button_wids[10]: P6_Select_Bias_Group, Event, 5
			button_wids[11]: P6_Select_Linearity_Group, Event, 4
			button_wids[12]: P6_Select_Bias_Group, Event, 6
			button_wids[13]: P6_Make_Image_Matches_Method_2, Event
			button_wids[14]: P6_View_Image_Match_Table_Method_2, Event
			button_wids[15]: P6_Combine_Image_Matches_Method_2, Event
			;
			button_wids[16]: P6_Read_Directory, Event
			button_wids[17]: P6_View_Coadded_Images, Event
			button_wids[18]: P6_Read_Coadded_Images, Event
			button_wids[19]: P6_Show_Pixel_Plot, Event
			;
			button_wids[20]: P6_Start_Linearity_Calculation, Event
			;
			else:	MSP_BDP_print_text, Event, "Error - P6 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;-----------------------------------------------------------------
pro P6_Text_Handler, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	event_type = STRUPCASE(TAG_NAMES(Event, /STRUCTURE_NAME))
	;
	;	x or y?
	;
	if(Event.id eq (*s_ptr).P6_text_wid[0]) then begin
		;print, "X window"
		ix = 1
	endif else begin
		;print, "Y window"
		ix = 0
	endelse
	;
	;
	;print, "Structure type = ",event_type
	;
	;help, Event, /structure
	;
	;	if text select or context, just get value from widget and
	;	copy it to structure and text box
	;
	if(event_type eq 'WIDGET_TEXT_SEL' or event_type eq 'WIDGET_CONTEXT') then begin
		WIDGET_CONTROL, Event.id, GET_VALUE=xval
		;
		;print, "  Text select or context - widget value = ",xval
		;
		;	clear the contents
		;
		if(ix) then (*l_ptr).xx = 0 else (*l_ptr).yy = 0
		;
		WIDGET_CONTROL, Event.id, SET_VALUE=''
	endif else begin
		;
		; this was character insert
		;
		;print, "  Char insert. Char = ",Event.ch
		;
		;	be sure it is  in the range 0-9, only
		;
		if(Event.ch gt 57B or Event.ch lt 48B and Event.ch ne 10B) then begin
			WIDGET_CONTROL, Event.id, SET_VALUE=''
			goto, skip_it
		endif
		;
		; get current value in window
		;
		WIDGET_CONTROL, Event.id, GET_VALUE=xval
		;print, "  current value = ",xval
		;print, "  current variable type = ",SIZE(xval, /TYPE)
		;
		;	load into buffer
		;
		zz = 0L
		reads, STRTRIM(xval,2), zz
		if(ix) then (*l_ptr).xx = zz else (*l_ptr).yy = zz
		;print, "  saved value = ",(*l_ptr).xx
		if(zz lt 0 or zz gt 1025) then begin
			if(ix) then (*l_ptr).xx = 0L else (*l_ptr).yy = 0L
			WIDGET_CONTROL, Event.id, SET_VALUE=''
			MSP_BDP_print_text, Event, "ERROR - pixel location is outside image"
		endif
		;
		skip_it:
	endelse
end
;
;----------------------------------------------------------------
pro P6_Tab_Handler, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	;	mostly just keep current tab current
	;
	ct = WIDGET_INFO((*s_ptr).P6_tab_wid, /TAB_CURRENT)
	(*s_ptr).P6_current_tab = ct
	;
	;	if current tab = 0 (Method 1), issue warning
	;
	if(ct eq 0) then MSP_BDP_print_text, Event, "WARNING - Method 1 deprecated - use Method 2"
	;
end
;-----------------------------------------------------------------
pro P7_Handler, Event
	;
	buttons = [	"P7_Droplist_Select_Night",$
				;
				"P7_Button_View_Edit_Groups",$
				;
				"P7_Button_Start_Image_Creation"$
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P7_Select_Night, Event
			;
			button_wids[1]: P7_View_Group_Table, Event
			;
			button_wids[2]: P7_Start_Image_Calculation, Event
			;
			else:	MSP_BDP_print_text, Event, "Error - P7 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end

;-----------------------------------------------------------------
pro P8_Handler, Event
	;
	buttons = [	"P8_Button_View_Linearity_Tables",$
				"P8_Droplist_Select_Night",$
				;
				"P8_Button_View_Edit_Groups",$
				;
				"P8_Button_Start_Linearity_Correction"$
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P8_View_Linearity_Tables, Event
			button_wids[1]: P8_Select_Night, Event
			;
			button_wids[2]: P8_View_Group_Table, Event
			;
			button_wids[3]: P8_Start_Linearity_Correction, Event
			;
			else:	MSP_BDP_print_text, Event, "Error - P8 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end

;-----------------------------------------------------------------
pro P9_Handler, Event
	;
	buttons = [	"P9_Button_Discover_Dark_Groups",$
				"P9_Button_Edit_Dark_Group_List",$
				"P9_Button_Compute_Dark_Images" $
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P9_Discover_Dark_Groups, Event
			button_wids[1]: P9_Edit_Group_Table, Event
			button_wids[2]: P9_Compute_Dark_Images, Event
			;
			else:	MSP_BDP_print_text, Event, "Error - P9 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;-----------------------------------------------------------------
pro P10_Handler, Event
	;
	buttons = [	"P10_Button_Photometric_Discover_Groups",$
				"P10_Button_Photometric_View_Edit_Flat_Groups",$
				"P10_Button_Photometric_Coadd_Group_Images",$
				"P10_Button_Photometric_Match_On_Off_Pairs",$
				"P10_Button_Photometric_View_On_Off_Matches",$
				"P10_Button_Photometric_Compute_Difference_Images",$
				"P10_Button_Photometric_Determine_Normalizations",$
				"P10_Button_Photometric_View_Normalizations",$
				"P10_Button_Photometric_Normalize",$
				;
				"P10_Button_Polarimetric_Discover_Groups",$
				"P10_Button_Polarimetric_View_Edit_Flat_Groups",$
				"P10_Button_Polarimetric_Coadd_Group_Images",$
				"P10_Button_Polarimetric_Match_On_Off_Pairs",$
				"P10_Button_Polarimetric_View_On_Off_Matches",$
				"P10_Button_Polarimetric_Compute_Difference_Images",$
				"P10_Button_Polarimetric_Determine_Normalizations",$
				"P10_Button_Polarimetric_View_Normalizations",$
				"P10_Button_Polarimetric_Normalize",$
				"P10_Button_Polarimetric_Do_Zero_Hop_Test",$
				"P10_Button_Polarimetric_Fix_Zero_Hops"$
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P10_Discover_Groups, Event, 'phot'
			button_wids[1]: P10_Edit_Flat_Groups, Event, 'phot'
			button_wids[2]: P10_Coadd_Groups, Event, 'phot'
			button_wids[3]: P10_Match_On_Off_Pairs, Event, 'phot'
			button_wids[4]: P10_View_On_Off_Pairs, Event, 'phot'
			button_wids[5]: P10_Compute_Difference_Images, Event, 'phot'
			button_wids[6]: P10_Determine_Normalizations, Event, 'phot'
			button_wids[7]: P10_View_Normalizations, Event, 'phot'
			button_wids[8]: P10_Normalize, Event, 'phot'
			;
			button_wids[9]:  P10_Discover_Groups, Event, 'pol'
			button_wids[10]: P10_Edit_Flat_Groups, Event, 'pol'
			button_wids[11]: P10_Coadd_Groups, Event, 'pol'
			button_wids[12]: P10_Match_On_Off_Pairs, Event, 'pol'
			button_wids[13]: P10_View_On_Off_Pairs, Event, 'pol'
			button_wids[14]: P10_Compute_Difference_Images, Event, 'pol'
			button_wids[15]: P10_Determine_Normalizations, Event, 'pol'
			button_wids[16]: P10_View_Normalizations, Event, 'pol'
			button_wids[17]: P10_Normalize, Event, 'pol'
			button_wids[18]: P10_Do_Zero_Hop_Test, Event, 'pol'
			button_wids[19]: P10_Fix_Zero_Hops, Event, 'pol'
			;
			else:	MSP_BDP_print_text, Event, "Error - P10 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;----------------------------------------------------------------
pro P10_Tab_Handler, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	;	mostly just keep current tab current
	;
	ct = WIDGET_INFO((*s_ptr).P10_tab_wid, /TAB_CURRENT)
	(*s_ptr).P10_current_tab = ct
	;
end
;
;-----------------------------------------------------------------
pro P11_Handler, Event
	;
	buttons = [	"P11_Button_Discover_Linearity_Images",$
				"P11_Button_View_Edit_Linearity_List",$
				"P11_Button_Linearity_Correct",$
				"P11_Button_Read_LC_Images",$
				"P11_Button_Compute_Read_Time_Image",$
				"P11_Button_Analyze_Read_Time_Image" $
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P11_Discover_Linearity_Images, Event
			button_wids[1]: P11_View_Edit_Linearity_List, Event
			button_wids[2]: P11_Linearity_Correct, Event
			button_wids[3]: P11_Read_LC_Images, Event
			button_wids[4]: P11_Compute_Read_Time_Image, Event
			button_wids[5]: P11_Analyze_Read_Time_Image, Event
			;
			else:	MSP_BDP_print_text, Event, "Error - P11 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;-----------------------------------------------------------------
pro P11_Text_Handler, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	rt_ptr = PTR_struct.PTR_RTime
	;
	;	if the text window is changed, update the structure variable
	;
	event_type = STRUPCASE(TAG_NAMES(Event, /STRUCTURE_NAME))
	;
	;	x or y?
	;
	if(Event.id eq (*s_ptr).P11_text_wid[0]) then begin
		;print, "X window"
		ix = 1
	endif
	;
	;print, "Structure type = ",event_type
	;
	;help, Event, /structure
	;
	;	if text select or context, just get value from widget and
	;	copy it to structure and text box
	;
	if(event_type eq 'WIDGET_TEXT_SEL' or event_type eq 'WIDGET_CONTEXT') then begin
		WIDGET_CONTROL, Event.id, GET_VALUE=xval
		;
		rtv = 0.0d0
		reads, xval, format='(F8.6)', rtv
		if(ix) then (*rt_ptr).read_time_value = rtv
		;
	endif else begin
		;
		; this was character insert
		;
		;	be sure it is  in the range 0-9, and dp only
		;
		if(Event.ch gt 57B or Event.ch lt 48B and Event.ch ne 10B and Event.ch ne 46B) then begin
			WIDGET_CONTROL, Event.id, SET_VALUE=''
			goto, skip_it
		endif
		;
		; get current value in window
		;
		WIDGET_CONTROL, Event.id, GET_VALUE=xval
		;print, "  current value = ",xval
		;print, "  current variable type = ",SIZE(xval, /TYPE)
		;
		;	load into buffer
		;
		rtv = 0.0d0
		reads, xval, format='(F8.6)', rtv
		if(ix) then (*rt_ptr).read_time_value = rtv
		;print, "  saved value = ",(*l_ptr).xx
		;
		skip_it:
	endelse
end
;
;-----------------------------------------------------------------
pro P12_Handler, Event
	;
	buttons = [	"P12_Droplist_Select_Night",$
				;
				"P12_Button_Phot_Spec_Select_Groups",$
				"P12_Button_Phot_Spec_View_Edit_Groups",$
				"P12_Button_Phot_Spec_Do_Corrections",$
				;
				"P12_Button_Pol_Spec_Select_Groups",$
				"P12_Button_Pol_Spec_View_Edit_Groups",$
				"P12_Button_Pol_Spec_Do_Corrections", $
				"P12_Button_Do_Rollback",$
				"P12_Button_No_Rollback",$
				"P12_Checkbox_Ignore_Zero_Hop_Tests", $
				"P12_Radio_Fix_By_Dither" $
		   	   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do	button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: P12_Select_Night, Event
			;
			button_wids[1]: P12_Select_Groups, Event, 'phot'
			button_wids[2]: P12_View_Edit_Group_Table, Event, 'phot'
			button_wids[3]: P12_Start_Image_Calculation, Event, 'phot'
			;
			button_wids[4]: P12_Select_Groups, Event, 'pol'
			button_wids[5]: P12_View_Edit_Group_Table, Event, 'pol'
			button_wids[6]: P12_Start_Image_Calculation, Event, 'pol'
			;
			button_wids[7]: P12_Image_Rollback, Event, 1
			button_wids[8]: P12_Image_Rollback, Event, 0
			;
			button_wids[9]: P12_Toggle_ZHT_Correct_Checkbox, Event
			;
			button_wids[10]:P12_Toggle_ZHT_Dither, Event
			;
			else:	MSP_BDP_print_text, Event, "Error - P12 button push not recognized"
	    endcase
	endif else MSP_BDP_print_text, Event, "No button pushed."
end
;
;-----------------------------------------------------------------
pro P12_Tab_Handler, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	;	mostly just keep current tab current
	;
	ct = WIDGET_INFO((*s_ptr).P12_tab_wid, /TAB_CURRENT)
	(*s_ptr).P12_current_tab = ct
	;
end
;
;------------------------------------------------------------------
pro Next_Handler, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	num_nights = (*s_ptr).num_nights
	;
	;	figure out which tab called the handler
	;
	tab_wid = WIDGET_INFO(Event.top, FIND_BY_UNAME="Big_Tab")
	current_tab = WIDGET_INFO(tab_wid, /TAB_CURRENT)
	;
	;	increment by one, if less than max, else reset to 0
	;
	max_tab = WIDGET_INFO(tab_wid, /TAB_NUMBER)
	if(current_tab le max_tab-2) then WIDGET_CONTROL, tab_wid, SET_TAB_CURRENT=current_tab+1 else $
		WIDGET_CONTROL, tab_wid, SET_TAB_CURRENT=0
	;
	current_tab = WIDGET_INFO(tab_wid, /TAB_CURRENT)
	;
	Tab_Init, Event, current_tab
	;
end

;-----------------------------------------------------------------
pro Help_Handler, Event
	;
	MSP_BDP_print_text, Event, "Not yet supported"
end
;-----------------------------------------------------------------

;   {WIDGET_TAB, ID:0L, TOP:0L, HANDLER:0L, TAB:0}

;-----------------------------------------------------------------
pro Tab_Handle, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	num_nights = (*s_ptr).num_nights
	;
	;	what to do when arriving at a tab
	;
	current_tab = Event.tab
	;
	Tab_Init, Event, current_tab
	;
end
;
;-------------------------------
pro Tab_Init, Event, current_tab
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	case current_tab of
		1:  begin
				MSP_BDP_print_text, Event, "Opened Part 1 - Analysis Run Setup"
			end
		2:  begin
				MSP_BDP_print_text, Event, "Opened Part 2 - Image & Group Directory Building"
				P2_droplist_populate, Event
				for i=0,9 do begin
					if(i le 3) then P2_LED_sensitize, Event, i, 0
				endfor
			end
		3:	begin
				MSP_BDP_print_text, Event, "Opened Part 3 - Kokopeli Crack Analsis & Repair"
				P3_droplist_populate, Event
				P3_button_sensitize, Event, 0, 0
				P3_LED_sensitize, Event, 0, 0
				;
				(*s_ptr).P3_column_flag = 1 - (*s_ptr).P3_column_flag
				P3_toggle_checkbox, Event
			end
		4:  begin
				MSP_BDP_print_text, Event, "Opened Part 4 - RAW1 Jump Detection & Repair"
				P4_droplist_populate, Event
				for i=0,12 do begin
					P4_button_sensitize, Event, i, 0
					if(i le 1) then P4_LED_sensitize, Event, i, 0
				endfor
			end
		5:  begin
				MSP_BDP_print_text, Event, "Opened Part 5 - Bad Pixel Mask Creation"
				P5_droplist_populate, Event
			end
		6:  begin
				MSP_BDP_print_text, Event, "Opened Part 6 - Manage Linearity Tables"
				P6_Init, Event
			end
		7:	begin
				MSP_BDP_print_text, Event, "Opened Part 7 - Make Ave RAW1 Images"
				P7_droplist_populate, Event
			end
		8:	begin
				MSP_BDP_print_text, Event, "Opened Part 8 - Apply Linearity Correction"
				P8_droplist_populate, Event
			end
		9:	begin
				MSP_BDP_print_text, Event, "Opened Part 9 - Make Dark Images"
				P9_init, Event
			end
		10:	begin
				MSP_BDP_print_text, Event, "Opened Part 10 - Make Flat Field Images"
				P10_init, Event
			end
		11:	begin
				MSP_BDP_print_text, Event, "Opened Part 11 - Measure Readout Time"
				P11_init, Event
			end
		12:	begin
				MSP_BDP_print_text, Event, "Opened Part 12 - Perform All Corrections"
				P12_init, Event
			end
		else: begin
			  end
	endcase
	;
end
