function kokepeli_maximum_detector, vector
;
;   slides a filter across the vector to help
;   find the maximum pixel without all the upset
;   of a single pixel search
;
;   the filter kernel is  -1, -1, -1, 2, 2, 2, -1, -1, -1
;
;   DPC 20050621 initial code (as minimum detector)
;     20050726 recast as maximum detector to work on convolved data
;
sz = n_elements(vector)
;
;print, "number of elements = ",sz
;
filter = [-1.,-1.,-1.,3.,5.,3.,-1.,-1.,-1.]
;
convolve = fltarr(sz)
mean = median_filtered_mean(vector)
;
;print, "In KMD, mean = ",mean[0]
;
;   for sz steps across vector
;
for i = 0, sz-1 do begin
    ;
    ;  set up the convolution
    ;
    weight = 0.
    sum = 0.
    ;
    for j = 0, 8 do begin
       k = (i + j - 4)
       if(k ge 0 and k le sz-1) then begin
         sum = sum + vector[k]*filter[j]
         weight = weight + filter[j]
       endif
    endfor
    ;
    ;  normalize by weight or replace with average (if no good data)
    ;
    if(sum ne 0.) then convolve[i] = sum / weight else convolve[i] = mean[0]
endfor
;
;   convolve done, now walk across finding maximum
;   skip first, last 3 channels - if maximum found there, something is wrong
;
zmax = -1.0e6
ymax = -1.0e6
;
for i = 3, sz-4 do begin
    if(convolve[i] gt zmax) then begin ; find peak
       zmax = convolve[i]
       ymax = i
       ;
       ;print, "max Y = ",ymax
       ;
    endif
endfor
;
;print, "Convolve = ",convolve
;
return, ymax
end

