function quad_shift, array, iquad, ishift
;
;   basic column, row shifter for single quads of Aladdin III images
;     (either CDS or RAW1 may be shifted, but note that they are
;      *not* mirrored - to shift a CDS *and* a RAW1 image, this
;      function must be called twice
;
;   inputs:
;      array = input array in memory
;      iquad = integer index of quad (0 = LL, 1 = LR, 2 = UR, 3 = UL)
;      ishift = integer identifier of type of shift
;               0 = move first or last column to middle, pushing others "outwards"
;               1 = move central column up one row, loading deleted data in center,
;                   and pushing top pixels "off"
;                >1 = reserved for future expansion
;
;   output (return value): copy of array after execution of indicated shift
;
;   DPC 20050623    original code (incomplete)
;       20050801    code completed
;       20060220    modified to work with sub-framed images
;
;-----------------------------------------------------------------------
;
;
;print, "entered quad_shift with iquad, ishift = ", iquad, ishift
result = array  ; type result, reserve space
;
;   determine array size
;
array_size = SIZE(array)
ncol = array_size[1]
nrow = array_size[2]
;
case iquad of
    3: begin          ; UL
         irow1 = (nrow)/2       ;    513    ;  old was 512
         irow2 = (nrow-1)       ; 1025       ; lower quads are 512 rows, uppers are 514 -- does
         row_end = irow2
         row_start = irow1               ;   this make sense? Check with Brian Taylor...
         irshift = -1               ; no - images show 513/513
         icol1 = 0
         icol2 = (ncol-2)/2     ;  511
         col_end = icol1
         col_start = icol2
         icshift = -1     ; shift left
       end
    2: begin          ; UR
         irow1 = (nrow)/2       ;    513    ;  old was 512
         irow2 = (nrow-1)       ; 1025
         row_end = irow2
         row_start = irow1
         irshift = -1
         icol1 = (ncol)/2       ;  512
         icol2 = (ncol-1)       ; 1023
         col_end = icol2
         col_start = icol1
         icshift = +1     ; shift right
       end
    0: begin          ; LL
         irow1 = 0
         irow2 = (nrow-2)/2     ;  512  ;  old was 511
         row_end = irow1
         row_start = irow2
         irshift = +1
         icol1 = 0
         icol2 = (ncol-2)/2     ;  511
         col_end = icol1
         col_start = icol2
         icshift = -1
       end
    1: begin          ; LR
         irow1 = 0
         irow2 = (nrow-2)/2     ;  512  ;  old was 511
         row_end = irow1
         row_start = irow2
         irshift = +1
         icol1 = (ncol)/2     ;  512
         icol2 = (ncol-1)     ; 1023
         col_end = icol2
         col_start = icol1
         icshift = +1
       end
endcase
;
col_save = fltarr(irow2-irow1+1)
;
case ishift of
    0: begin   ; de-wrap column
    ;
          ;print, "Doing column de-wrap"
          ;
          ;   shifting engine
          ;
          colmax = (ncol-4)/2     ;  510
          col_save = reform(result[col_end,irow1:irow2])     ; save the end column
          for icol = 0, colmax do begin
              current_col = col_start + icol*icshift
              result[current_col+icshift,irow1:irow2] = array[current_col,irow1:irow2]
          endfor
          result[col_start,irow1:irow2] = col_save
       end
    1: begin    ; push up the starting column one row
          ;print, "Doing column push with col_start, irow2, irow1 = ",col_start, irow2, irow1
          ;
          nrows = irow2 - irow1 + 1
        for irow = 0, nrows-2 do begin
            row = row_end + irow*irshift
           result[col_start,row] = result[col_start,row+irshift]
        endfor
        result[col_start,row_start] = 0
       end
    else: begin        ; expansion
           xx = 0.
          end
endcase
;
;     done
;
return, result
end


